/*---------------------------------------------------------------------------*/
/* Program:  average.c                                                       */
/*                                                                           */
/* Purpose:  This program reads in a sequence of images and calculates       */
/*           the average pixel value at each (x,y) location.  The images     */
/*           all need to be the same size.                                   */
/*                                                                           */
/* Author:   John Gauch, Edu Metz                                            */
/*                                                                           */
/* Date:     June 15, 1996 - Original program.                               */
/*           July 23, 1996 - Color support added.                            */
/*                                                                           */
/* Note:     Copyright (C) The University of Kansas, 1996                    */
/*---------------------------------------------------------------------------*/
#include <IM.h>

/* Program variables */
int Debug = FALSE;

/* Image variables */
char Name1[50];
char Name2[50];
IM_TYPE *Image1;
IM_TYPE *Image2;
int PixType, Xdim, Ydim, Zdim, DimCnt;
int NewXdim, NewYdim;

/*---------------------------------------------------------------------------*/
/* Purpose:  This routine calculates the average pixel value of grey images. */
/*---------------------------------------------------------------------------*/
void GreyAverage(int argc, char *argv[], int i)
{
   FLOAT_TYPE **Data1;
   SHORT_TYPE **Data2;
   int j, x, y;

   /* Get file name of first image */
   if (sscanf(argv[i], "%s", Name2) == 0)
      Error("Could not get input file name");

   /* Read first image */
   Image2 = im_open(Name2, &PixType, &Xdim, &Ydim, &Zdim, &DimCnt);
   Data2 = (SHORT_TYPE **) im_alloc2D(Image2, SHORT);
   im_read(Image2, SHORT, (char *) &(Data2[0][0]));

   /* Create output image */
   Image1 = im_create(Name1, PixType, NewXdim, NewYdim, 1);
   Data1 = (FLOAT_TYPE **) im_alloc2D(Image1, FLOAT);

   if (Debug == TRUE)
      printf("Processing %s\n", Name2);

   /* Copy first image */
   for (y = 0; y < Ydim; y++)
      for (x = 0; x < Xdim; x++)
	 Data1[y][x] = (float) Data2[y][x];

   /* Process remaining images in sequence */
   for (j = i + 1; j < argc; j++)
   {

      /* Get file name of next image */
      if (sscanf(argv[j], "%s", Name2) == 0)
	 Error("Could not get input file name");

      /* Read next image */
      Image2 = im_open(Name2, &PixType, &Xdim, &Ydim, &Zdim, &DimCnt);
      im_read(Image2, SHORT, (char *) &(Data2[0][0]));

      if (Debug == TRUE)
	 printf("Processing %s\n", Name2);

      /* Add next image */
      for (y = 0; y < Ydim; y++)
	 for (x = 0; x < Xdim; x++)
	    Data1[y][x] += (float) Data2[y][x];
   }

   /* Normalize output image */
   for (y = 0; y < Ydim; y++)
      for (x = 0; x < Xdim; x++)
	 Data1[y][x] /= (float) (argc - i);

   /* Write information to output image */
   im_write(Image1, FLOAT, (char *) &(Data1[0][0]));

   im_free2D((char **) Data1);
   im_free2D((char **) Data2);
}

/*---------------------------------------------------------------------------*/
/* Purpose:  This routine calculates the average pixel value of color images. */
/*---------------------------------------------------------------------------*/
void ColorAverage(int argc, char *argv[], int i)
{
   COLOR_TYPE **Data1;
   COLOR_TYPE **Data2;
   int j, x, y, Temp;

   /* Get file name of first image */
   if (sscanf(argv[i], "%s", Name2) == 0)
      Error("Could not get input file name");

   /* Read first image */
   Image2 = im_open(Name2, &PixType, &Xdim, &Ydim, &Zdim, &DimCnt);
   Data2 = (COLOR_TYPE **) im_alloc2D(Image2, COLOR);
   im_read(Image2, COLOR, (char *) &(Data2[0][0]));

   /* Create output image */
   Image1 = im_create(Name1, PixType, NewXdim, NewYdim, 1);
   Data1 = (COLOR_TYPE **) im_alloc2D(Image1, COLOR);

   if (Debug == TRUE)
      printf("Processing %s\n", Name2);

   /* Copy first image */
   for (y = 0; y < Ydim; y++)
      for (x = 0; x < Xdim; x++)
      {
	 Data1[y][x].r = Data2[y][x].r;
	 Data1[y][x].g = Data2[y][x].g;
	 Data1[y][x].b = Data2[y][x].b;
      }

   /* Process remaining images in sequence */
   for (j = i + 1; j < argc; j++)
   {

      /* Get file name of next image */
      if (sscanf(argv[j], "%s", Name2) == 0)
	 Error("Could not get input file name");

      /* Read next image */
      Image2 = im_open(Name2, &PixType, &Xdim, &Ydim, &Zdim, &DimCnt);
      im_read(Image2, COLOR, (char *) &(Data2[0][0]));

      if (Debug == TRUE)
	 printf("Processing %s\n", Name2);

      /* Add next image */
      for (y = 0; y < Ydim; y++)
	 for (x = 0; x < Xdim; x++)
	 {
	    Temp = Data1[y][x].r + Data2[y][x].r;
	    if (Temp > 255)
	       Temp = 255;
	    Data1[y][x].r = Temp;
	    Temp = Data1[y][x].g + Data2[y][x].g;
	    if (Temp > 255)
	       Temp = 255;
	    Data1[y][x].g = Temp;
	    Temp = Data1[y][x].b + Data2[y][x].b;
	    if (Temp > 255)
	       Temp = 255;
	    Data1[y][x].b = Temp;
	 }
   }

   /* Normalize output image */
   for (y = 0; y < Ydim; y++)
      for (x = 0; x < Xdim; x++)
      {
	 Data1[y][x].r /= (argc - i);
	 Data1[y][x].g /= (argc - i);
	 Data1[y][x].b /= (argc - i);
      }

   /* Write information to output image */
   im_write(Image1, COLOR, (char *) &(Data1[0][0]));

   im_free2D((char **) Data1);
   im_free2D((char **) Data2);
}

/*---------------------------------------------------------------------------*/
/* Purpose:  This is the main program.                                       */
/*---------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
/* Program variables */
   int i = 0, j;

   /* Interpret program options */
   printf("AVERAGE Program - KUIM Version 2.0\n\n");
   while ((++i < argc) && (argv[i][0] == '-'))
      switch (argv[i][1])
      {
      case 'd':
	 Debug = TRUE;
	 break;
      default:
	 Error("Invalid option encountered");
	 break;
      }

   /* Check number of file names */
   if (argc - i < 2)
   {
      fprintf(stderr, "Usage: average [options] outfile infile ...\n");
      fprintf(stderr, "       [-d]    Print debugging information\n");
      exit(1);
   }

   /* Get output file names from argument list */
   if (sscanf(argv[i++], "%s", Name1) == 0)
      Error("Could not get output file name");

   /* Check dimensions of input images */
   for (j = i; j < argc; j++)
   {

      if (sscanf(argv[j], "%s", Name2) == 0)
	 Error("Could not get input file name");

      Image2 = im_open(Name2, &PixType, &Xdim, &Ydim, &Zdim, &DimCnt);

      if (DimCnt != 2)
	 Error("Can only process 2D images");

      if (j == i)
      {
	 NewXdim = Xdim;
	 NewYdim = Ydim;
      }
      else if (NewXdim != Xdim || NewYdim != Ydim)
	 Error("Input image dimensions do not match");
   }

   /* Process input images */
   if (PixType == COLOR || PixType == PSEUDO || PixType == JPEG_RGB)
      ColorAverage(argc, argv, i);
   else
      GreyAverage(argc, argv, i);
   return (0);
}
