/*---------------------------------------------------------------------------*/
/* Program:  diff_contour.c                                                  */
/*                                                                           */
/* Purpose:  This program calculates the mean Euclidean distance between     */
/*           the points on two contours.  These points are expected to be    */
/*           in ascii files with one (x,y) coordinate one per line. Each     */
/*           contour starts with "START CONTOUR" and ends with "-1 -1".      */
/*                                                                           */
/* Author:   John Gauch                                                      */
/*                                                                           */
/* Date:     May 4, 1994.                                                    */
/*                                                                           */
/* Note:     Copyright (C) The University of Kansas, 1994                    */
/*---------------------------------------------------------------------------*/
#include <IM.h>

/*---------------------------------------------------------------------------*/
/* Purpose:  This is the main program.                                       */
/*---------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
   /* Program variables */
   int Debug = FALSE;
   char Name1[50];
   char Name2[50];
   FILE *fd1;
   FILE *fd2;
   int X1[5000], Y1[5000];
   int X2[5000], Y2[5000];
   int Count1 = 0, Count2 = 0;
   int i = 0, j, x, y;
   int dx, dy, dist, mindist;
   float avedist;

   /* Interpret program options */
   printf("DIFF_CONTOUR Program - KUIM Version 1.0\n\n");
   while ((++i < argc) && (argv[i][0] == '-'))
      switch (argv[i][1])
      {
      case 'd':
	 Debug = TRUE;
	 break;
      default:
	 Error("Invalid option encountered");
	 break;
      }

   /* Check number of file names */
   if (argc - i != 2)
   {
      fprintf(stderr, "Usage: diff_contour [options] contour1 contour2\n");
      fprintf(stderr, "       [-d]    Print debugging information\n");
      exit(1);
   }

   /* Get image file names from argument list */
   if (sscanf(argv[i++], "%s", Name1) == 0)
      Error("Could not get contour1 file name");
   if (sscanf(argv[i++], "%s", Name2) == 0)
      Error("Could not get contour2 file name");

   /* Read contour1 file */
   if ((fd1 = fopen(Name1, "r")) == NULL)
      Error("Could not open contour1 file");
   if (fscanf(fd1, "START CONTOUR\n") == EOF)
      Error("Could not read contour1 file");
   while ((fscanf(fd1, "%d %d\n", &x, &y) != EOF) && (x >= 0) && (y >= 0))
   {
      X1[Count1] = x;
      Y1[Count1] = y;
      Count1++;
   }
   fclose(fd1);
   if (Debug == TRUE)
      printf("Contour 1 has %d points\n", Count1);

   /* Read contour2 file */
   if ((fd2 = fopen(Name2, "r")) == NULL)
      Error("Could not open contour2 file");
   if (fscanf(fd2, "START CONTOUR\n") == EOF)
      Error("Could not read contour2 file");
   while ((fscanf(fd2, "%d %d\n", &x, &y) != EOF) && (x >= 0) && (y >= 0))
   {
      X2[Count2] = x;
      Y2[Count2] = y;
      Count2++;
   }
   fclose(fd2);
   if (Debug == TRUE)
      printf("Contour 2 has %d points\n", Count2);

   /* Calculate distance between contours */
   for (i = 0; i < Count1; i++)
   {
      dx = X1[i] - X2[0];
      dy = Y1[i] - Y2[0];
      mindist = dx * dx + dy * dy;
      for (j = 1; j < Count2; j++)
      {
	 dx = X1[i] - X2[j];
	 dy = Y1[i] - Y2[j];
	 dist = dx * dx + dy * dy;
	 if (mindist > dist)
	    mindist = dist;
      }
      if (Debug == TRUE)
	 printf("min %d was %d\n", i, mindist);
      avedist += (float) sqrt((double) mindist);
   }
   avedist /= Count1;
   printf("Average Euclidean distance = %f\n", avedist);

   /* Exit program */
   return (0);
}
