/*---------------------------------------------------------------------------*/
/* Program:  interp_contour.c                                                */
/*                                                                           */
/* Purpose:  This program aligns two contours and uses linear interpolation  */
/*           to obtain intermediate contours.  Points are expected to be     */
/*           in ascii files with one (x,y) coordinate one per line. Each     */
/*           contour starts with "START CONTOUR" and ends with "-1 -1".      */
/*                                                                           */
/* Author:   John Gauch                                                      */
/*                                                                           */
/* Date:     June 3, 1994                                                    */
/*                                                                           */
/* Note:     Copyright (C) The University of Kansas, 1994                    */
/*---------------------------------------------------------------------------*/
#include <IM.h>
#define SIZE 1000

/*---------------------------------------------------------------------------*/
/* Purpose:  This is the main program.                                       */
/*---------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
   /* Program variables */
   int Debug = TRUE;
   char Name1[50];
   char Name2[50];
   char Name3[50];
   FILE *fd1;
   FILE *fd2;
   FILE *fd3;
   int X1[SIZE], Y1[SIZE];
   int X2[SIZE], Y2[SIZE];
   int Count1 = 0, Count2 = 0;
   int i = 0, x, y, i1, i2, dx, dy, dist, mindist, minshift;

   /* Interpret program options */
   printf("INTERP_CONTOUR Program - KUIM Version 1.0\n\n");
   while ((++i < argc) && (argv[i][0] == '-'))
      switch (argv[i][1])
      {
      case 'd':
	 Debug = TRUE;
	 break;
      default:
	 Error("Invalid option encountered");
	 break;
      }

   /* Check number of file names */
   if (argc - i != 3)
   {
      fprintf(stderr,
	      "Usage: interp_contour [options] contour1 contour2 outfile\n");
      fprintf(stderr, "       [-d]    Print debugging information\n");
      exit(1);
   }

   /* Get image file names from argument list */
   if (sscanf(argv[i++], "%s", Name1) == 0)
      Error("Could not get contour1 file name");
   if (sscanf(argv[i++], "%s", Name2) == 0)
      Error("Could not get contour2 file name");
   if (sscanf(argv[i++], "%s", Name3) == 0)
      Error("Could not get output file name");

   /* Read contour1 file */
   if ((fd1 = fopen(Name1, "r")) == NULL)
      Error("Could not open contour1 file");
   if (fscanf(fd1, "START CONTOUR\n") == EOF)
      Error("Could not read contour1 file");
   while ((fscanf(fd1, "%d %d\n", &x, &y) != EOF) && (x >= 0) && (y >= 0))
   {
      X1[Count1] = x;
      Y1[Count1] = y;
      Count1++;
   }
   fclose(fd1);
   if (Debug == TRUE)
      printf("Contour 1 has %d points\n", Count1);

   /* Read contour2 file */
   if ((fd2 = fopen(Name2, "r")) == NULL)
      Error("Could not open contour2 file");
   if (fscanf(fd2, "START CONTOUR\n") == EOF)
      Error("Could not read contour2 file");
   while ((fscanf(fd2, "%d %d\n", &x, &y) != EOF) && (x >= 0) && (y >= 0))
   {
      X2[Count2] = x;
      Y2[Count2] = y;
      Count2++;
   }
   fclose(fd2);
   if (Debug == TRUE)
      printf("Contour 2 has %d points\n", Count2);

   /* Find best alignment of two contours */
   mindist = SIZE * SIZE;
   for (i = 0; i < Count1; i++)
   {
      dist = 0;
      for (i1 = 0; i1 < Count1; i1++)
      {
	 i2 = (i1 * Count2 / Count1 + i) % Count2;
	 dx = X1[i1] - X2[i2];
	 dy = Y1[i1] - Y2[i2];
	 dist += dx * dx + dy * dy;
      }
      if (dist < mindist)
      {
	 mindist = dist;
	 minshift = i;
      }
   }
   if (Debug == TRUE)
      printf("minshift %d mindist %d\n", minshift, mindist);

   /* Create interpolated contour */
   if ((fd3 = fopen(Name3, "w")) == NULL)
      Error("Could not create output file");
   fprintf(fd3, "START CONTOUR\n");
   for (i1 = 0; i1 < Count1; i1++)
   {
      i2 = (i1 * Count2 / Count1 + minshift) % Count2;
      x = (X1[i1] + X2[i2]) / 2;
      y = (Y1[i1] + Y2[i2]) / 2;
      fprintf(fd3, "%d %d\n", x, y);
   }
   fprintf(fd3, "-1 -1\n");
   fclose(fd3);

   /* Exit program */
   return (0);
}
