/*---------------------------------------------------------------------------*/
/* Program:  make_raw.c                                                      */
/*                                                                           */
/* Purpose:  This program strips off the 1024 byte header from a KUIM        */
/*           image to get a raw pixel file.                                  */
/*                                                                           */
/* Author:   John Gauch                                                      */
/*                                                                           */
/* Date:     January 23, 1995 - Original program.                            */
/*           August 1, 1997   - Added support for ascii.                     */
/*                                                                           */
/* Note:     Copyright (C) The University of Kansas, 1995                    */
/*---------------------------------------------------------------------------*/
#include <IM.h>
#define ASCII 42
#define RAW   24

/*---------------------------------------------------------------------------*/
/* Purpose:  This is the main program.                                       */
/*---------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
   /* Image variables */
   char Name1[50];
   char Name2[50];
   IM_TYPE *Image;
   SHORT_TYPE *Data;
   int PixType = RAW, Xdim, Ydim, Zdim, DimCnt;

   /* Global variables */
   int Debug = FALSE;
   FILE *Fd1;
   FILE *Fd2;
   int i = 0;
   int Byte, Count;

   /* Interpret program options */
   printf("MAKE_RAW Program - KUIM Version 2.0\n\n");
   while ((++i < argc) && (argv[i][0] == '-'))
      switch (argv[i][1])
      {
      case 'A':
	 PixType = ASCII;
	 break;
      case 'd':
	 Debug = TRUE;
	 break;

      default:
	 Error("Invalid option encountered");
	 break;
      }

   /* Check number of file names */
   if (argc - i != 2)
   {
      fprintf(stderr, "Usage: make_raw [options] infile outfile\n");
      fprintf(stderr, "       [-d]  Print debugging information\n");
      fprintf(stderr, "       [-A]  Output ASCII file\n");
      exit(1);
   }

   /* Get image file names from argument list */
   if (sscanf(argv[i++], "%s", Name1) == 0)
      Error("Could not get input file name");
   if (sscanf(argv[i++], "%s", Name2) == 0)
      Error("Could not get output file name");

   /* Handle RAW file output */
   if (PixType == RAW)
   {
      /* Open files */
      Fd1 = fopen(Name1, "r");
      Fd2 = fopen(Name2, "w");

      /* Copy input image pixels */
      for (i = 0; i < 1024; i++)
	 Byte = fgetc(Fd1);
      while ((Byte = fgetc(Fd1)) != EOF)
	 fputc((char) Byte, Fd2);

      /* Close files */
      fclose(Fd1);
      fclose(Fd2);
   }

   /* Handle ASCII file output */
   else if (PixType == ASCII)
   {
      /* Read input image */
      Image = im_open(Name1, &PixType, &Xdim, &Ydim, &Zdim, &DimCnt);
      Data = (SHORT_TYPE *) im_alloc1D(Image, SHORT);
      im_read(Image, SHORT, (char *) &(Data[0]));

      /* Write output file */
      Fd2 = fopen(Name2, "w");
      fprintf(Fd2, "%3d %3d %3d ", Xdim, Ydim, Zdim);
      Count = Xdim * Ydim * Zdim;
      for (i = 0; i < Count; i++)
	 if (i % 20 == 0)
	    fprintf(Fd2, "\n%3d ", Data[i]);
	 else
	    fprintf(Fd2, "%3d ", Data[i]);
      fclose(Fd2);
   }
   return (0);
}
